@@ -342,8 +342,9 @@ class cSubstanceEAPnl(wxgSubstanceEAPnl.wxgSubstanceEAPnl, gmEditArea.cGenericEd
 			return False
 
 		loincs = self._LCTRL_loincs.item_data
-		if len(loincs) > 0:
-			subst.loincs = loincs
+		if loincs is not None:
+			if len(loincs) > 0:
+				subst.loincs = loincs
 
 		self.data = subst
 		return True
@@ -362,8 +363,9 @@ class cSubstanceEAPnl(wxgSubstanceEAPnl.wxgSubstanceEAPnl, gmEditArea.cGenericEd
 			return False
 
 		loincs = self._LCTRL_loincs.item_data
-		if len(loincs) > 0:
-			self.data.loincs = loincs
+		if loincs is not None:
+			if len(loincs) > 0:
+				self.data.loincs = loincs
 
 		return True
 
@@ -633,9 +635,10 @@ class cSubstanceDoseEAPnl(wxgSubstanceDoseEAPnl.wxgSubstanceDoseEAPnl, gmEditAre
 
 	#----------------------------------------------------------------
 	def _save_as_new(self):
+		pk_subst = self._PRW_substance.GetData()
 		dose = gmMedication.create_substance_dose (
-			pk_substance = self._PRW_substance.GetData(),
-			substance = self._PRW_substance.GetValue().strip(),
+			pk_substance = pk_subst,
+			substance = self._PRW_substance.GetValue().strip() if (pk_subst is None) else None,
 			amount = decimal.Decimal(self._TCTRL_amount.GetValue().strip().replace(',', '.')),
 			unit = gmTools.coalesce(self._PRW_unit.GetData(), self._PRW_unit.GetValue().strip(), function_initial = ('strip', None)),
 			dose_unit = gmTools.coalesce(self._PRW_dose_unit.GetData(), self._PRW_dose_unit.GetValue().strip(), function_initial = ('strip', None))
@@ -1120,6 +1123,8 @@ class cDrugProductEAPnl(wxgDrugProductEAPnl.wxgDrugProductEAPnl, gmEditArea.cGen
 		wxgDrugProductEAPnl.wxgDrugProductEAPnl.__init__(self, *args, **kwargs)
 		gmEditArea.cGenericEditAreaMixin.__init__(self)
 
+		self.__component_doses = []
+
 		self.mode = 'new'
 		self.data = data
 		if data is not None:
@@ -1202,13 +1207,13 @@ class cDrugProductEAPnl(wxgDrugProductEAPnl.wxgDrugProductEAPnl, gmEditArea.cGen
 
 	#----------------------------------------------------------------
 	def _save_as_new(self):
-
 		drug = gmMedication.create_drug_product (
 			product_name = self._PRW_product_name.GetValue().strip(),
 			preparation = gmTools.coalesce (
 				self._PRW_preparation.GetData(),
 				self._PRW_preparation.GetValue()
 			).strip(),
+			doses = self.__component_doses if (len(self.__component_doses) > 0) else None,
 			return_existing = True
 		)
 		drug['is_fake_product'] = self._CHBOX_is_fake.GetValue()
@@ -1217,14 +1222,8 @@ class cDrugProductEAPnl(wxgDrugProductEAPnl.wxgDrugProductEAPnl, gmEditArea.cGen
 		if code != u'':
 			drug['external_code'] = code
 			drug['external_code_type'] = self._PRW_external_code_type.GetData().strip()
-
 		drug.save()
-
-		if len(self.__component_doses) > 0:
-			drug.set_substance_doses_as_components(substance_doses = self.__component_doses)
-
 		self.data = drug
-
 		return True
 
 	#----------------------------------------------------------------
@@ -1260,7 +1259,7 @@ class cDrugProductEAPnl(wxgDrugProductEAPnl.wxgDrugProductEAPnl, gmEditArea.cGen
 
 		self._PRW_product_name.SetFocus()
 
-		self.__component_substances = []
+		self.__component_doses = []
 
 	#----------------------------------------------------------------
 	def _refresh_as_new_from_existing(self):
@@ -1562,7 +1561,6 @@ class cSingleComponentGenericDrugEAPnl(wxgSingleComponentGenericDrugEAPnl.wxgSin
 
 	#----------------------------------------------------------------
 	def _save_as_new(self):
-
 		dose = gmMedication.create_substance_dose (
 			pk_substance = self._PRW_substance.GetData(),
 			amount = self._TCTRL_amount.Value.strip(),
